/*
 *  Doku, siehe http://www.mikrocontroller.net/articles/AVR_FAT32
 *  Neuste Version: http://www.mikrocontroller.net/svnbrowser/avr-fat32/
 *  Autor: Daniel R.
 */

#ifndef _FILE_H
  #define _FILE_H
  #include <stdio.h>
  
  // Please note that the whole avrfat32 library is not htread safe.
  // It can only open and process a single file at a time.
  // Therefore special care is required when used within a web server.
  
  // Open an existing file or create a new file.
  // The rw_flag can be either 'r' for read, 'w' for write, or 'c' for create.
  extern uint8_t  ffopen( uint8_t name[], uint8_t rw_flag);
  
  // Read a byte from the open file (the library buffers in blocks of 512 bytes)
  extern uint8_t  ffread(void);
  
  // Write a byte into the open file.
  extern void     ffwrite( uint8_t c);
  
  // Write a string into the open file.
  extern void     ffwrites( uint8_t *s );
  
  // Write multiple bytes into the open file.
  extern void     ffwriten( uint8_t *s, uint16_t n ); 
  
  // Close the file. Must be called before opening another file.
  extern uint8_t  ffclose(void);
  
  // Seek to a position withing the open file. Next read/write will happen at that position.
  extern void     ffseek(uint32_t offset);
  
  // Change into a sub directory.
  extern uint8_t  ffcd( uint8_t name[]);
  
  // Change back to the parent directory.
  extern uint8_t  ffcdLower(void);      
  
  // List the content of the current directory.
  // The stream parameter is normally stdout.
  extern void     ffls(FILE* stream);
  
  // Remove a file
  extern uint8_t  ffrm( uint8_t name[]);
  
  // Create a sub-directory in the current directory.
  extern void     ffmkdir( uint8_t name[]);
  
  // Update file information, write outstanding data.
  extern void     fflushFileData(void);
  
  // Check if a file exists.
  extern uint8_t  ffileExists( uint8_t name[]);

#endif
  