/*
 *         Doku, siehe http://www.mikrocontroller.net/articles/AVR_FAT32
 *         Latest version: http://www.mikrocontroller.net/svnbrowser/avr-fat32/
 *         Autor: Daniel R.
 */


#ifndef CONFIG_H_
        #define CONFIG_H_

        #include <ctype.h>
        #include <stdint.h>
        #include <avr/io.h>
        
        #define TRUE  0x01
        #define FALSE 0x00

        // result of ffopen()
        #define MMC_FILE_OPENED  0  // file exists and opened
        #define MMC_FILE_CREATED 1  // new file created
        #define MMC_FILE_ERROR   2  // file does not exist or cannot be created.


        //==================================================
        // Enable or Disable features
        //==================================================
        
        // Support writing files.
        #define MMC_WRITE               FALSE
        
        // Allow overwriting existing files, requires MMC_WRITE=TRUE.
        #define MMC_OVER_WRITE          FALSE
        
        // Speed up writing by using multi-block operations. 
        // May not work on all cards and requires MMC_OVER_WRITE=FALSE.
        // This feature is normally disabled.
        #define MMC_MULTI_BLOCK         FALSE
        
        // Bit order, must be true on AVR.
        #define MMC_ENDIANNESS_LITTLE   TRUE
        
        // Support long filenames (as introduced with Windows 95).
        // Costs about 800 bytes program memory.
        // The ffls() function displays only the short filenames anyway.
        #define MMC_LFN_SUPPORT         FALSE
        
        // Support removing files and directories with ffrm().
        #define MMC_RM                  FALSE
        
        // Restrict ffrm() to remove only files but no directories.
        #define MMC_RM_FILES_ONLY       TRUE

        // Support gettings file timestamps with fat_getTime() .
        #define MMC_TIME_STAMP          FALSE

        // Support seeking withing files with ffseek().
        #define MMC_SEEK                FALSE
        
        // Support creating directories with ffmkdir(), requires MMC_WRITE=TRUE.
        #define MMC_MKDIR               FALSE
        
        // Support getting free space of medium with fat_getFreeBytes().
        #define MMC_GET_FREE_BYTES      FALSE
        
        // Support listing directories with ffls().
        #define MMC_LS                  TRUE
        
        // Support changing the working directory with ffcd() and ffcdLower().
        #define MMC_CD                  FALSE
        
        // Support checking whether a file exists with ffFileExists().
        #define MMC_FILE_EXSISTS        FALSE
        
        // Support writing Strings into a file with ffwrites().
        #define MMC_WRITE_STRING        FALSE
        
        // Support writing multple characters into a file with ffwriten().
        #define MMC_WRITEN              FALSE

        // Maximum number of clusters to read/write without fat-lookup, if the media is not fragmented.
        // Be careful, MMC_MAX_CLUSTERS_IN_ROW*fat.secPerClust must fit into a short integer!
        #define MMC_MAX_CLUSTERS_IN_ROW 256


        //===============================
        // SPI settings
        //===============================
        
        // Enbale switching to maximum speed after card initialisation.
        // This is normally TRUE, the value FALSE migth help debugging with a logic-analyzer.
        #define MMC_MAX_SPEED          TRUE

        // Use software SPI instead of hardware SPI port. All pins must be on the same port.
        // Note: Might not work, it has not been tested.
        #define MMC_SOFT_SPI           FALSE

        //==============================
        // Data structures
        //==============================

        extern struct Fat_t {             // fat daten (1.cluster, root-dir, dir usw.)
                uint32_t lastSector;      // nicht der aktuell geladene sektor, sondern der davor
                uint8_t  bufferDirty;     // puffer wurde beschrieben, sector muss geschrieben werden bevor er neu geladen wird
                uint32_t dir;             // Direktory zeiger rootDir=='0' sonst(1.Cluster des dir; start auf root)
                uint32_t rootDir;         // Sektor(f16)/Cluster(f32) nr root directory
                uint32_t dataDirSec;      // Sektor nr data area
                uint32_t fatSec;          // Sektor nr fat area
                uint8_t  secPerClust;     // anzahl der sektoren pro cluster
                uint8_t  fatType;         // fat16 oder fat32 (16 oder 32)
                uint8_t  card_type;       // der SD/MMC karten typ. siehe auch mmc.c
                uint8_t  sector[512];     // der puffer fuer sektoren !
        } fat;

        extern struct File_t {            // datei infos usw.
                uint16_t cntOfBytes;      // seek + cntOfBytes ist position in datei
                uint32_t seek;            // seek + cntOfBytes ist position in datei
                uint32_t currentSectorNr; // aktuell zu bearbeitender sektor bei einer geoeffneten datei
                uint32_t length;          // 28,4 datei Laenge (4-byte)
                uint8_t  *name;           // zeiger auf langen dateinamen (lfn)...
                                          // ueber den wird auch im ordner nach der datei gesucht. 
                                          // der kurze dateiname (sfn) leitet sich aus dem lfn ab!
                uint8_t  row;             // reihe im sektor in der die datei infos stehen (reihe 0-15)
                uint32_t firstCluster;    // 20,2 /26,2        datei 1.cluster hi,low(moeglicherweise der einzige) (4-byte)
                uint32_t entrySector;     // der sektor in dem der sfn dateieintrag ist.
        } file;

        extern struct Chain_t {
                uint32_t startSectors;    // erster freier/verketteter sektor
                uint16_t cntSecs;         // anzahl der freien/verketteten in einer reihe
                uint32_t lastCluster;     // letzter bekannter cluster der teilkette davor !
        } chain;


#endif
