#include <avr/interrupt.h>
#include <avr/io.h>
#include "../uip/clock.h"
#include "../uip/timer.h"

// The system clock counts up every 1ms
// It is used by uIP to perform time related stuff.

static volatile int timercounter;

#ifdef CARDSLOT
    static uint8_t mmcTimerPrescaler;
    volatile uint8_t TimingDelay;    // For mmc.c. Counts down every 100ms
#endif

// on ATmega targets
#ifdef TCCR0A
    ISR(TIMER0_COMPA_vect)
    {
        timercounter++;
        
        #ifdef CARDSLOT
        if (++mmcTimerPrescaler>=100) {
            mmcTimerPrescaler=0;
            TimingDelay = (TimingDelay==0) ? 0 : TimingDelay-1;
        }
        #endif
    }

    void clock_init(void)
    {
        cli();
        TCCR0A = 0x02;
        TCCR0B = 0x05;
        OCR0A = F_CPU/1024000; 
        TIMSK0 = 0x02;
        sei();
    }
#endif
    
// on Atxmega targets
#ifdef TCC0
    ISR(TCC0_OVF_vect) {
        timercounter++;
        
        #ifdef CARDSLOT
        if (++mmcTimerPrescaler>=100) {
            mmcTimerPrescaler=0;
            TimingDelay = (TimingDelay==0) ? 0 : TimingDelay-1;
        }
        #endif        
    }

    void clock_init(void)
    {
        cli();
        PMIC.CTRL |= (1<<PIN0)+(1<<PIN1)+(1<<PIN2);
        TCC0.CNT = 0x00;
        TCC0.PER =  31250;
        TCC0.CTRLA = 0x01;
        TCC0.CTRLB = 0x00;
        TCC0.INTCTRLA = 0x03;
        sei();
    }
#endif

// This function is called by uIP to query the system time in seconds
int clock_time(void)
{
    return(timercounter);
}
