#ifndef __SERIALCONSOLE_H_
#define __SERIALCONSOLE_H_

#include <stdint.h>

// The serial bitrate and CPU clock frequency is usually set in the Makefile
// #define SERIAL_BITRATE 115200
// #define F_CPU 20000000

// In the software, lines are always terminated with \n.
// If terminal mode is enabled, line breaks are converted automatically:
// - in sending direction, line breaks are sent as \r\n.
// - in receiving direction, line breaks my be terminated by \r, \n or \r\n.
// - backspace characters in receiving direction delete the last received character from the input buffer.
#define TERMINAL_MODE 1

// Enable echo on serial interface
#define SERIAL_ECHO 1

// Size of input buffer in chars, max 254 (output is unbuffered)
#define SERIAL_INPUT_BUFFER_SIZE 80

// In case of ATmega targets, the following definition may be set to use the second serial port instead of the first one:
// #define USE_SERIAL2

// In case of ATxmega targets, the following definitions are required:
// #define SERIALPORT USARTE0
// #define SERIALPORT_RXC_vect USARTE0_RXC_vect

// Initialize the serial port and bind it to stdin and stdout.
// Don't forget to set the TxD pin to output mode in case of ATxmega.
void initserial(void);

// Returns 1 if the serial port has received at least one character
// so that following calls to getc() do not block.
uint8_t receivedChar(void);

// Returns 1 if the serial port has received a line break
// so that following calls to gets() and scanf() do not block.
uint8_t receivedLine(void);

#endif //__SERIALCONSOLE_H_
