#ifndef _shiftregisters_H__
#define _shiftregisters_H__
#include <stdint.h>

/*
 * You need to add settings like the following to hw-layout.h:
 * 
 *   // Number of shift register inputs and outputs (0, 8, 16, 24 or 32)
 *   #define XL_INPUTS            24
 *   #define XL_OUTPUTS           24
 *   
 *   // Macros for inputs (Port X)
 *   #define SHIFTin              { writeBit(PORTD,2,1); writeBit(PORTD,2,0); }
 *   #define DATAin               readBit(PIND,3)
 *   #define LOADin               { writeBit(PORTD,4,0); writeBit(PORTD,4,1); } 
 *   
 *   // Macros for outputs (Port Y)
 *   #define STOREout             { writeBit(PORTD,5,1); writeBit(PORTD,5,0); }
 *   #define SHIFTout             { writeBit(PORTD,6,1); writeBit(PORTD,6,0); }
 *   #define DATAout(value)       writeBit(PORTD,7,value)
 */


#ifdef SHIFTin
    // Read inputs of shift registers
    // 8-32 bits, provided by 1-4 shift registers of type 74HC165
    uint32_t read_XL_inputs(void);
#endif


#ifdef SHIFTout
    // Current status of the extension outputs (port X)
    extern uint32_t xlOutputState;
    
    // Write outputs of shift registers
    // 8-32 bits, provided by 1-4 shift registers of type 74HC595
    void write_XL_outputs(const uint32_t value);
#endif

#endif // _shiftregisters_H__